<?php
/**
 * Created by PhpStorm.
 * User: vladislavdavarasvili
 * Date: 13/10/2017
 * Time: 15:51
 */

class Form {
	private $db;

	function __construct() {

		try {
			$this->db = new PDO( 'mysql:host=localhost;dbname=seven;charset=utf8mb4', 'root', 'root' );
			$this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		}catch(PDOException $exception){
			throw new Exception($exception->getMessage());
		}
	}

	function createForm($json){

		$this->db->beginTransaction();

		try {
			$stmt = $this->db->prepare("INSERT INTO form (form_name, form_created, form_modified) VALUES (:name, :created, :modified)");
			$stmt->bindParam(':name', $json->formName);
			$time = time();
			$stmt->bindParam(':created', $time);
			$stmt->bindParam(':modified', $time);


			$stmt->execute();


			$formId = $this->db->lastInsertId();

			foreach ($json->formDetails as $detailKey=>$formDetail) {
				//echo "key:".$detailKey.'<BR>';
				$stmt = $this->db->prepare("INSERT INTO form_details (form_id,form_key,form_label,form_type,form_min,form_max,form_required,form_mask,form_default,form_checked) VALUES(:form_id,:form_key,:form_label,:form_type,:form_min,:form_max,:form_required,:form_mask,:form_default,:form_checked)");
				$stmt->bindParam(':form_id',$formId);
				$stmt->bindParam(':form_key',$detailKey);
				$stmt->bindParam(':form_label',$formDetail->label);
				$stmt->bindParam(':form_type',$formDetail->type);
				$stmt->bindParam(':form_min',$formDetail->min);
				$stmt->bindParam(':form_max',$formDetail->max);
				$isRequired = (($formDetail->required)?1:0);
				$stmt->bindParam(':form_required',$isRequired);

				$isMask = (($formDetail->mask!='')?$formDetail->mask:'');
				$stmt->bindParam(':form_mask',$isMask);

				$stmt->bindParam(':form_default',$formDetail->defaultValue);
				$checkedValue = (($formDetail->checkedValue != '')?$formDetail->checkedValue:'');
				$stmt->bindParam(':form_checked',$checkedValue);
				$stmt->execute();

				if($formDetail->type == 'enum'){
					//->list
					$detailId = $this->db->lastInsertId();
					foreach ($formDetail->list as $listKey=>$listValue){
						$stmt = $this->db->prepare("INSERT INTO form_options (detail_id,option_key,option_value) VALUES(:detail_id,:option_key,:option_value)");
						$stmt->bindParam(':detail_id',$detailId);
						$stmt->bindParam(':option_key',$listKey);
						$stmt->bindParam(':option_value',$listValue);
						$stmt->execute();
					}
				}
			}



			foreach($json->displayDetails as $displayKey=>$formDisplay){
				$stmt = $this->db->prepare("INSERT INTO form_display (form_id,form_label,form_fields,form_key) VALUES(:form_id,:form_label,:form_fields,:form_key)");
				$stmt->bindParam(':form_id',$formId);
				$stmt->bindParam(':form_key',$displayKey);
				$stmt->bindParam(':form_fields',$formDisplay->fieldList);
				$stmt->bindParam(':form_label',$formDisplay->label);
				$stmt->execute();
			}

			$this->db->commit();

			return true;
		}catch (Exception $exception){
			throw new Exception("Can not save form details. <BR>".$exception->getMessage().'<BR> file:'.$exception->getFile().'<BR> code:'.$exception->getCode().'<BR> '.$exception->getTraceAsString().'<BR> '.$exception->getTrace().'<BR> '.$exception->getLine());
		}


	}

	function getFormList(){
		$sql= "SELECT form.*,(SELECT COUNT(form_content.id) FROM form_content WHERE form_content.form_id=form.form_id AND form_content.content_id=0) as totalRecords FROM form  ORDER BY form.form_created DESC";
		$stmt = $this->db->prepare($sql);
		$stmt->execute();
		return $stmt->fetchAll(PDO::FETCH_ASSOC);
	}

	function getFormById($formId, $contentId = 0){
		$stmt = $this->db->prepare("SELECT * FROM form_details WHERE form_id=:formId");
		$stmt->bindParam(':formId',$formId);
		$stmt->execute();
		$formDetails = $stmt->fetchAll(PDO::FETCH_ASSOC);

		$formObject = array();

		$returnFormDetails = array();
		foreach ($formDetails as $form_detail){
			if($form_detail['form_type'] == 'enum'){
				$stmt = $this->db->prepare("SELECT * FROM form_options WHERE detail_id=:detailId");
				$stmt->bindParam(':detailId',$form_detail['detail_id']);
				$stmt->execute();
				$form_detail['options'] = $stmt->fetchAll(PDO::FETCH_ASSOC);
			}
			$returnFormDetails[$form_detail['detail_id']]=$form_detail;
		}

		$formObject['form'] = $returnFormDetails;

		if($contentId>0) {
			$cStmt = $this->db->prepare( "SELECT * FROM form_content WHERE id=:content_id OR (content_id=0 AND content_id=:content_id)");
			$cStmt->bindParam( ':content_id', round( $contentId ) );
			$cStmt->execute();
			$contentList = $cStmt->fetchAll(PDO::FETCH_ASSOC);

			$contentArray = array();
			foreach ($contentList as $content){
				$contentArray[$content['detail_id']] = $content;
			}

			$formObject['content'] = $contentArray;
		}

		return $formObject;
	}



	function saveFormDetails($formId,$formDetails,$postContent){
		//should check formId exist in table 'form'
		//$formId = $formDetails[0]['form_id'];
		/**@Todo: validate form data via formDetails array  */
		/**
		 * formDetails[detail_id] = _POST[key]
		 */
		$this->db->beginTransaction();

		try {
			$recordId = $this->db->lastInsertId();
			foreach ($formDetails['form'] as $form_detail){
				$item = $postContent[$form_detail['detail_id']];

				$stmt = $this->db->prepare("INSERT INTO form_content(form_id,content_id,detail_id,form_value) VALUES(:form_id,:content_id,:detail_id,:form_value)");
				$stmt->bindParam(':form_id',$formId);
				$stmt->bindParam(':content_id',$recordId);
				$stmt->bindParam(':detail_id',$form_detail['detail_id']);
				if($form_detail['form_type'] == 'bool'){
					$item = round($item);
				}
				$stmt->bindParam(':form_value',$item);
				$stmt->execute();
				if($recordId == 0){
					$recordId = $this->db->lastInsertId();
				}
			}

			$this->db->commit();


			return true;
		}catch (Exception $e){
			throw new Exception("Can not save form");
		}

	}

	/* contentId can be removed if using current way to update */
	function updateFormDetails($formDetails,$postContent,$contentId=0){


		$this->db->beginTransaction();

		try {

			foreach ($formDetails['form'] as $detailId=>$form_detail){


				$item = $postContent[$form_detail['detail_id']];

				/** Can be done like this, or in other way, see below. depends on indexes and structure */
				/*
				$stmt = $this->db->prepare("UPDATE form_content SET form_value=:form_value WHERE ((id=:content_id AND content_id=0) OR (content_id=:content_id)) AND detail_id=:detail_id");
				$stmt->bindParam(':content_id',$contentId);
				$stmt->bindParam(':detail_id',$form_detail['detail_id']);
				*/
				$stmt = $this->db->prepare("UPDATE form_content SET form_value=:form_value WHERE id=:id");

				if($form_detail['form_type'] == 'bool'){
					$item = round($item);
				}
				$stmt->bindParam(':id',$formDetails['content'][$detailId]['id']);
				$stmt->bindParam(':form_value',$item);
				$stmt->execute();

			}

			$this->db->commit();


			return true;
		}catch (Exception $e){
			throw new Exception("Can not save form");
		}
	}


	function getFormContent($formId){
		$stmt = $this->db->prepare("SELECT * FROM form_display WHERE form_id=:form_id");
		$stmt->bindParam(':form_id',$formId);
		try{
			$stmt->execute();
			$formDisplayParams = $stmt->fetchAll(2);



		}catch (Exception $exception){
			throw new Exception("Can not get form display style");
		}
	}
}